function ODS = apply_sparse_reconstruction(Spectrum,SPECP,FreqIdx,ANTP,ALGOP,VibroANTP)
%% Sparse reconstruction using a regular microphone array
% INPUT PARAMETERS
%       Spectra
%       SPECP
%       FreqIdx
%       ANTP
%       ALGOP  = structure with type & parameters of the algorithm to use
%
% OUTPUT PARAMETERS
%       ODS = Map of the recovered velocities


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Nearfield ACoustic HOlography with Sparse regularization (NACHOS)
% Version 1.0                                    
%     
% Copyright 2012 Antoine Peillot, Franois Ollivier, Gilles Chardon, 
%                Laurent Daudet, Nancy Bertin, Rmi Gribonval
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Dx = ALGOP.DicoSizeX;
Dy = ALGOP.DicoSizeY;

% Nr of points on the vibrating plate

Nx = length(unique(VibroANTP.XYPos(:,1)));
Ny = length(unique(VibroANTP.XYPos(:,2)));

Mx = ceil((ALGOP.DicoSizeX-Nx)/2);
My = ceil((ALGOP.DicoSizeY-Ny)/2);


% Mask over the plate
ValFreq = SPECP.FreqValues(FreqIdx);
mask = zeros(Dx,Dy);
mask(Mx+1:Dx-Mx,My+1:Dy-My) = ALGOP.Mask;

CsSig = Spectrum(:);
Coords = [ANTP.XYPos repmat(ANTP.ZPos,length(ANTP.XYPos),1)];
CsMes = [CsSig Coords];

% Transfer Matrix Computation
k = ValFreq*2*pi/ANTP.SoundCelerity; % Wave number
[phi, phi_norm] = build_measurement_matrix(k, CsMes(:,2:4), ALGOP, @(x) rfft2(x), mask);

if strcmpi(ALGOP.Name, 'l1_constr_spgl1')
    nCS = norm(CsMes(:,1));
    Epsilon = nCS * ALGOP.StoppingCriterion / 100; % constraint: error rate
    options = spgSetParms('verbosity',1);
    FFT2ODS = spg_bpdn(phi, CsMes(:,1), Epsilon,options);
else
    addpath(genpath('toolbox/sparsify_0_4'));
    Iter = ALGOP.StoppingCriterion;
    FFT2ODS = greed_omp(squeeze(CsMes(:,1)), phi_norm, Dx*Dy, 'stopCrit', 'M', 'stopTol', Iter, 'verbose', false);
end

% Applying Inverse Fourier transform and reshaping
FFT2ODS = reshape(FFT2ODS, Dx, Dy);
ODS = irfft2(FFT2ODS);
ODS = ODS(Mx+1:Dx-Mx,My+1:Dy-My);
