function ODS = apply_tikhonov_reconstruction(Spectrum,SPECP,FreqIdx,REGANTP,VibroANTP,nfft)
%% Standard NAH reconstruction with Tikhonov regularization using a regular microphone array
% INPUT PARAMETER
% ValFreq = Chosen frequency 
% OUTPUT PARAMETER
% ODS = Map of the recovered velocities

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Nearfield ACoustic HOlography with Sparse regularization (NACHOS)
% Version 1.0                                    
%     
% Copyright 2012 Antoine Peillot, Franois Ollivier, Gilles Chardon, 
%                Laurent Daudet, Nancy Bertin, Rmi Gribonval
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Frequency selection

FFT2ODS = NAH_Tikhonov(Spectrum,SPECP,FreqIdx,REGANTP,nfft);
ODS = ifft2(ifftshift(FFT2ODS));

VibroXPos = unique(VibroANTP.XYPos(:,1));
VibroYPos = unique(VibroANTP.XYPos(:,2));

RegXPos = unique(REGANTP.XYPos(:,1));
RegYPos = unique(REGANTP.XYPos(:,2));

Nbx = length(RegXPos);
Nby = length(RegYPos);

ODS = ODS(1:Nbx,1:Nby); 

% interpolate on vibrometry points (interpolation is performed on ODS.'
% because meshgrid matrices X and Y that are transposed compared to our
% initial coordinate convention

[X Y] = meshgrid(RegXPos,RegYPos); 
[XI YI] = meshgrid(VibroXPos,VibroYPos);
ODS = interp2(X, Y, ODS.', XI, YI,'spline');
ODS = ODS.';


function kvsa = NAH_Tikhonov(Spectrum, SPECP, FreqIdx, REGANTP, nfft)
%% Tikhonov Regularization
% Spectrum = Hologram
% Freq = Chosen frequency
% xH,yH = coordinates of the microphones
% step = step between microphones (=4 when 120 microphones are used among
% the 1920)

Nbx = length(unique(REGANTP.XYPos(:,1)));
Nby = length(unique(REGANTP.XYPos(:,2)));

Freq = SPECP.FreqValues(FreqIdx);

%% Constantes
C = REGANTP.SoundCelerity;
RhoC = C * 1.204; % from Fourier Acoustics

% Size of the hologram

dx = max(REGANTP.XYPos(2,1)-REGANTP.XYPos(1,1),REGANTP.XYPos(2,2)-REGANTP.XYPos(1,2)); 

dk = 2*pi/(nfft*dx);
kmax = pi / dx;
kx = linspace(-kmax, kmax-dk, nfft);
ky = kx;

kx2 = ones(nfft,1)*kx.^2;
ky2 = ky'.^2*ones(1, nfft);
k = 2*pi*Freq/C;
kz = sqrt(k^2 - kx2 - ky2);


Spectrum = reshape(Spectrum,Nby,Nbx); % BOUSTROPHEDON
Spectrum = Spectrum.';

%% Smoothing the edges of the hologram plane: Tukey window
% it was decided not to use this window in the results presented in the
% paper for simplicity and readibility reason; uncomment if you want to try

% Lx = REGANTP.XLength;
% Ly = REGANTP.YLength;
% 
% W = 0.2; % TODO
% xw = Lx*W;
% yw = Ly*W;
% Wx = logical(abs(XH)>(Lx/2-xw));
% Tx = 1-Wx.*(0.5+0.5.*cos(pi.*(abs(XH)-Lx/2)./xw));
% Wy = logical(abs(YH)>(Ly/2-yw));
% Ty = 1-Wy.*(0.5+0.5.*cos(pi.*(abs(YH)-Ly/2)./yw));
% Spectrum = Spectrum.*Tx.*Ty;

%% spatial Fourier transform
kSpectrum =  fftshift(fft2(Spectrum, nfft, nfft));

%% Spectrum of the velocity on the hologram plane
kSpectrumf = kSpectrum.*kz/RhoC/k;

d = REGANTP.ZPos; % distance between the source plane and the hologram plane
G = exp(1i.*kz.*d); % Green function
GG = conj(G).*G;

lbd = 0.001 : 0.001 : 2;
L = numel(lbd);
X = zeros(L,1);
Egcv = zeros(L,1);
H = zeros(nfft,nfft,L);

% GCV function
for ii = 1 : L
    Ginv = conj(G) ./ (GG + lbd(ii).*(lbd(ii)./(lbd(ii)+GG)).^2);
    kvsa = Ginv .* kSpectrumf;
    Res = kvsa.*G - kSpectrumf;
    X(ii) = norm(Res(:));
    H(:,:,ii) = GG ./ (GG + lbd(ii).*(lbd(ii)./(lbd(ii)+GG)).^2);
    Egcv(ii) = X(ii)^2 ./ (trace(1 - H(:,:,ii)))^2;
end

% Optimal regularization: minimum of the GCV function estimation
Ilbd = Egcv == min(Egcv);
lbdopt = lbd(Ilbd);
Hopt = conj(G) ./ (GG + lbdopt.*(lbdopt./(lbdopt+GG)).^2);
kvsa = Hopt .* kSpectrumf;