function [Spectra SPECP] = load_spectra_d(input_filename,Fmax)
% Load a measurement file
% Usage
%   [Spectra SPECP] = load_spectra(input_filename)
%
% Input: 
%   input_filename      path to the file to be loaded
%   Fmax                maximum frequency (in Hertz) kept in the spectra
%
% Output: 
%   Spectra             data matrix of size 
%                       SPECP.NbChannels x SPECP.NbFreqs
%   SPECP.SamplingFreq  physical sampling frequency in Hertz
%   SPECP.NbSamples     number of time samples per channel
%   SPECP.NbChannels    number of channels
%   SPECP.Nfft          size of the FFT used to compute the spectra
%   SPECP.df            resolution in Hertz of the bin size
%   SPECP.FreqMax       = Fmax
%   SPECP.NbFreqs       number of frequency bins kept
%   SPECP.FreqValues    values of the NbFreqs kept between 0 and Fmax
%   SPECP.FreqIndices   1:NbFreqs
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Nearfield ACoustic HOlography with Sparse regularization (NACHOS)
% Version 1.0                                    
%     
% Copyright 2012 Antoine Peillot, Franois Ollivier, Gilles Chardon, 
%                Laurent Daudet, Nancy Bertin, Rmi Gribonval
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%% File opening
File = fopen(input_filename, 'rb');

if File==-1
    error('File not found');
end

%% Header reading
% Sampling frequency
SPECP.SamplingFreq = fread(File, 1, 'int32');
% Number of samples per signal
SPECP.NbSamples = fread(File, 1, 'int32');
% Number of microphones
SPECP.NbChannels = fread(File, 1, 'int32');

%% Constants needed for spectral analysis
% Length of the fft
SPECP.Nfft = 2^nextpow2(SPECP.NbSamples);
% Resolution
SPECP.df = SPECP.SamplingFreq/(SPECP.Nfft-1);
% Frequency range
SPECP.FreqMax    = Fmax;
SPECP.NbFreqs = round(Fmax / SPECP.df);
SPECP.FreqValues = (0 : SPECP.NbFreqs-1)' * SPECP.df;
SPECP.FreqIndices  = 1:SPECP.NbFreqs;

%% Content reading & spectral analysis
disp(['Allocating ' num2str(SPECP.NbChannels) ' x ' num2str(SPECP.NbFreqs) ' matrix']);
Spectra = zeros(SPECP.NbChannels,SPECP.NbFreqs);
disp('Starting to fill it');
for ii = 1:SPECP.NbChannels
    disp(['Filling channel ' num2str(ii) '/' num2str(SPECP.NbChannels)]);
    % Loading excitation signal
    MeasuredSignal = fread(File, SPECP.NbSamples, 'single');
    % Loading measured signal
    ExcitationSignal   = fread(File, SPECP.NbSamples, 'single');
    %% Spectral analysis
    % Excitation spectrum
    temp = 1/sqrt(SPECP.Nfft) .* fft(ExcitationSignal,SPECP.Nfft);
    ExcitationSpectrum = temp(1:SPECP.NbFreqs);
    % Referenced signal spectrum
    temp = 1/sqrt(SPECP.Nfft) .* fft(MeasuredSignal, SPECP.Nfft);
    Spectra(ii,:) = temp(1:SPECP.NbFreqs) .* conj(ExcitationSpectrum) ./ abs(ExcitationSpectrum);
    Spectra(isnan(Spectra(ii,:))) = 0;
end
Spectra(find(isnan(Spectra))) = eps;
disp('Done');
fclose(File);




