function [Spectrum,ANTP] = select_datasubset(Spectra,ANTP,FreqIdx,OuterFlag,DownSampleType,DownSampleParam)

% from the full data (all spectra from all measurement points in ANTP),
% subselects one frequency and a subset of the original sensor set

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Nearfield ACoustic HOlography with Sparse regularization (NACHOS)
% Version 1.0                                    
%     
% Copyright 2012 Antoine Peillot, Franois Ollivier, Gilles Chardon, 
%                Laurent Daudet, Nancy Bertin, Rmi Gribonval
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Spectrum = Spectra(:,FreqIdx);

switch DownSampleType
    
    case 'none'
         if OuterFlag
            [Spectrum,ANTP] = remove_outer_microphones(Spectrum,ANTP);    
         end
        
    case 'regular'
        % the antenna must have a regular structure
        
        % check regularity of the antenna
        if ~strcmp(ANTP.SensorDisposition,'regular')
            error('Unable to regularly downsample an irregular antenna');
        end
        
        % check that sensor spacing is the same in both directions
        if DownSampleParam(2)~=DownSampleParam(5)
            error('Cannot downsample differently in x and y')
        end    
        
        XPos = unique(ANTP.XYPos(:,1));
        YPos = unique(ANTP.XYPos(:,2));
        
        SelXPos = XPos(DownSampleParam(1):DownSampleParam(2):DownSampleParam(3));
        SelYPos = YPos(DownSampleParam(4):DownSampleParam(5):DownSampleParam(6));
        
        SelectionIdxMask = ones(ANTP.NbSensors,1);
        
        for ii=1:ANTP.NbSensors         
            if isempty(find((ANTP.XYPos(ii,1)==SelXPos),1)) || isempty(find((ANTP.XYPos(ii,2)==SelYPos),1))
                SelectionIdxMask(ii) = 0;
            end
        end
            
        IdxToKeep = find(SelectionIdxMask);
        
        ANTP.XYPos = ANTP.XYPos(IdxToKeep,:);
        Spectrum = Spectrum(IdxToKeep);
        
        ANTP.NbSensors = length(IdxToKeep);
        
    case 'random'
        % the antenna can be of any type
        
        if OuterFlag        
            [Spectrum,ANTP] = remove_outer_microphones(Spectrum,ANTP);       
        end
        
        permutation = randperm(ANTP.NbSensors);
        RandomlySelectedIndices = permutation(1:DownSampleParam);
        ANTP.XYPos = ANTP.XYPos(RandomlySelectedIndices,:);
        ANTP.NbSensors  = size(ANTP.XYPos,1);
        Spectrum = Spectrum(RandomlySelectedIndices);  
end


function [Spectrum,ANTP] = remove_outer_microphones(Spectrum,ANTP)

xmin = ANTP.PlateBoundingBox(1);
xmax = ANTP.PlateBoundingBox(2);
ymin = ANTP.PlateBoundingBox(3);
ymax = ANTP.PlateBoundingBox(4);

ANTP.OuterMicroIndices = find((ANTP.XYPos(:,1)<=xmin) + ...
    (ANTP.XYPos(:,1)>=xmax) + ...
    (ANTP.XYPos(:,2)<=ymin) + ...
    (ANTP.XYPos(:,2)>=ymax));

% remove them from antenna description
ANTP.XYPos(ANTP.OuterMicroIndices,:) = [];
ANTP.NbSensors  = size(ANTP.XYPos,1);

% remove from spectrum
Spectrum(ANTP.OuterMicroIndices) =  [];
