%% Script to reproduce Figure 3

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Nearfield ACoustic HOlography with Sparse regularization (NACHOS)
% Version 1.0                                    
%     
% Copyright 2012 Antoine Peillot, Franois Ollivier, Gilles Chardon, 
%                Laurent Daudet, Nancy Bertin, Rmi Gribonval
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% constants 

% number of points we wish to reconstruct on the plate
Nx = length(unique(VibroANTP.XYPos(:,1)));
Ny = length(unique(VibroANTP.XYPos(:,2)));

% Reconstruction with BPDN, from regular antenna measurements
ALGOP.Name = 'l1_constr_spgl1';
ALGOP.StoppingCriterion = 27;
ALGOP.DicoSizeX = 2*Nx;
ALGOP.DicoSizeY = 2*Ny;
ALGOP.StepX = 0.01;
ALGOP.StepY = 0.01;
ALGOP.DicoXLength = ALGOP.DicoSizeX*ALGOP.StepX;
ALGOP.DicoYLength = ALGOP.DicoSizeY*ALGOP.StepY;
ALGOP.Mask = ones(Nx,Ny);

NbMethods = 6;

ODSToPlot = cell(NbMethods,length(SPECPVibro.SelectedFreqIndices));

Correlations = zeros(NbMethods,length(SPECPVibro.SelectedFreqIndices));

%% Loop on modes to be imaged
for ii = 1:length(SPECPVibro.SelectedFreqIndices)
    
    % Prepare groundtruth image
    ODSFreqIdx = SPECPVibro.SelectedFreqIndices(ii);
    ODS        = SelectedGroundTruthODS(:,:,ii);
    
    Correlations(1,ii) = 1;
    
    % Reconstruction with Tikhonov reg. from regular antenna measurements
    nfft = 128; % the fft2d is 128x128 
    DownSampleParam = [1,1,48,1,1,40];
    [Spectrum, REGANTP1920] = select_datasubset(SpectraRegAnt, RegANTP, ODSFreqIdx,0,'none',DownSampleParam);  
    ODS_RegAnt1920_Tikhonov  = apply_tikhonov_reconstruction(Spectrum,SPECPRegAnt,ODSFreqIdx,REGANTP1920,VibroANTP,nfft);
    Correlations(2,ii) = compute_correlation_2d(ODS,ODS_RegAnt1920_Tikhonov);
    
    DownSampleParam = [3,4,47,3,4,39];
    [Spectrum, REGANTP120] = select_datasubset(SpectraRegAnt, RegANTP, ODSFreqIdx,0,'regular',DownSampleParam);
    ODS_RegAnt120_Tikhonov  = apply_tikhonov_reconstruction(Spectrum,SPECPRegAnt,ODSFreqIdx,REGANTP120,VibroANTP,nfft);
    Correlations(3,ii) = compute_correlation_2d(ODS,ODS_RegAnt120_Tikhonov);
    
    % Sparse Reconstructions from regular antenna measurements (120 micros in and out the plate)
    ODS_RegAnt120_BPDN = apply_sparse_reconstruction(Spectrum,SPECPRegAnt,ODSFreqIdx,REGANTP120, ALGOP,VibroANTP);
    Correlations(4,ii) = compute_correlation_2d(ODS,ODS_RegAnt120_BPDN);
    
    % Sparse Reconstructions from regular antenna measurements (120 micros
    % above plate)
    DownSampleParam = [7,3,40,3,3,32];
    [Spectrum, REGANTP120p] = select_datasubset(SpectraRegAnt, RegANTP, ODSFreqIdx,1,'regular',DownSampleParam);
    ODS_RegAnt120p_BPDN = apply_sparse_reconstruction(Spectrum,SPECPRegAnt,ODSFreqIdx,REGANTP120p, ALGOP,VibroANTP);
    Correlations(5,ii) = compute_correlation_2d(ODS,ODS_RegAnt120p_BPDN);
    
    % Reconstruction with BPDN, from rand antenna measurements
    DownSampleParam = 120;
    [Spectrum,RANDANTP120] = select_datasubset(SpectraRegAnt, RegANTP,ODSFreqIdx,1,'random',DownSampleParam);
    ODS_RandAnt120_BPDN = apply_sparse_reconstruction(Spectrum,SPECPRegAnt,ODSFreqIdx,RANDANTP120, ALGOP,VibroANTP);
    Correlations(6,ii) = compute_correlation_2d(ODS,ODS_RandAnt120_BPDN);
           
    % store for display
    ODSToPlot{1,ii} = flipud(Cpx2Surf(ODS));
    ODSToPlot{2,ii} = flipud(ODS_RegAnt1920_Tikhonov);
    ODSToPlot{3,ii} = flipud(ODS_RegAnt120_Tikhonov);
    ODSToPlot{4,ii} = flipud(ODS_RegAnt120_BPDN);
    ODSToPlot{5,ii} = flipud(ODS_RegAnt120p_BPDN);
    ODSToPlot{6,ii} = flipud(ODS_RandAnt120_BPDN);   
end

% create figure 3
f3=figure(3);
set(f3,'OuterPosition',[500, 500, 1000, 1000]);
plot_ODS(ODSToPlot,Correlations,ALGOP.Mask);

subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,1);
title('(a) Groundtruth');
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,2);
title('(b) TikhREG1920');
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,3);
title('(c) TikhREG120');
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,4);
title('(d) L1REG120');
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,5);
title('(e) L1REG120in');
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,6);
title('(f) L1RANfREG120in');

subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,1);
ylabel([num2str(round(SPECPVibro.FreqValues(SPECPVibro.SelectedFreqIndices(1)))),' Hz']);
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,7);
ylabel([num2str(round(SPECPVibro.FreqValues(SPECPVibro.SelectedFreqIndices(2)))),' Hz']);
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,13);
ylabel([num2str(round(SPECPVibro.FreqValues(SPECPVibro.SelectedFreqIndices(3)))),' Hz']);
subplot(length(SPECPVibro.SelectedFreqIndices),NbMethods,19);
ylabel([num2str(round(SPECPVibro.FreqValues(SPECPVibro.SelectedFreqIndices(4)))),' Hz']);
